package cn.iocoder.yudao.module.shipment.listener;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentLoadExcelExportPushEvent2;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.number.NumberUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.ecw.dal.dataobject.cabinet.CabinetDO;
import cn.iocoder.yudao.module.ecw.service.cabinet.CabinetService;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderGuanlian.OrderGuanlianDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseIn.OrderWarehouseInDO;
import cn.iocoder.yudao.module.order.dto.OrderBackInfoDto;
import cn.iocoder.yudao.module.order.service.order.OrderGuanlianService;
import cn.iocoder.yudao.module.order.service.order.OrderItemService;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.order.vo.order.OrderWarehouseInDetailsVO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrank.ProductBrankDO;
import cn.iocoder.yudao.module.product.service.product.ProductAttrService;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.product.service.productbrank.ProductBrankService;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dto.LoadDto;
import cn.iocoder.yudao.module.shipment.enums.CustomsTypeEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.utils.CustomCellWriteHandler;
import cn.iocoder.yudao.module.shipment.utils.CustomLoadMergeStrategyNew;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.WriteTable;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.HAVE_NOT_PRELOAD_GOODS;


/**
 * 自编号订单excel导出监听
 *
 * @author zhengYi
 */
@Component("ShipmentLoadExcelExportListener2")
@AllArgsConstructor
@Slf4j
public class ShipmentLoadExcelExportListener2 {

    private final OrderQueryService orderQueryService;
    private UeProperties ueProperties;
    private FileService fileService;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private BoxService boxService;

    @Resource
    private CabinetService cabinetService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private ProductBrankService productBrankService;

    @Resource
    private BoxApprovalService boxApprovalService;

    @Resource
    private OrderGuanlianService orderGuanlianService;

    @Resource
    private OrderWarehouseInService orderWarehouseInService;

    @Resource
    private OrderService orderService;

    @Resource
    private ProductService productService;

    @Resource
    private ProductAttrService productAttrService;

    private static List<Integer> colorList = new ArrayList<>();
    static {
        colorList.add(22);
        colorList.add(41);
        colorList.add(42);
        colorList.add(43);
        colorList.add(44);
        colorList.add(45);
        colorList.add(46);
        colorList.add(47);
    }

//    @Value("${ue.templates-url}")
//    private String path;

    /**
     * 订单导出监听
     *
     * @param event 编号记录
     */
    @EventListener(ShipmentLoadExcelExportPushEvent2.class)
    public void shipmentLoadExcelExportPushEvent(ShipmentLoadExcelExportPushEvent2 event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/load/excel/");
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("shipment_load.xlsx");
                InputStream inputStream = null;
                inputStream = getClass().getClassLoader().getResourceAsStream(ueProperties.getTemplatesUrl() + "/load.xlsx");
                Map headMap = new HashMap();
                JSONObject jsonObject = JSONObject.parseObject(event.getRequestParams());
                Long shipmentId = jsonObject.getLong("shipmentId");
                List<BoxPreloadGoodsBackVO> goodsItemList = boxPreloadGoodsService.getShipGoodItemList(shipmentId);
                if (CollectionUtil.isEmpty(goodsItemList)) {
                    event.setResult(HAVE_NOT_PRELOAD_GOODS.getMsg());
                }
                BoxDO box = boxService.getBox(shipmentId);
                CabinetDO cabinet = cabinetService.getCabinet(box.getCabinetId());
                BoxApprovalDO boxApprovalDO = boxApprovalService.getOne(new LambdaQueryWrapperX<BoxApprovalDO>().eq(BoxApprovalDO::getShipmentId, shipmentId).eq(BoxApprovalDO::getApprovalType, 2).last("limit 1"));
                headMap.put("selfNo", box.getSelfNo());
                headMap.put("ctnNo",box.getCubNo());
                if(cabinet != null) {
                    headMap.put("cabinetName", cabinet.getName());
                }
                headMap.put("exportDate", DateUtils.formatDate(new Date()));
                if (boxApprovalDO != null) {
                    headMap.put("date", DateUtils.formatDate(boxApprovalDO.getCreateTime()));
                }
                File fileDir = new File(dir);
                if (!fileDir.exists()){
                    // 不存在则创建一个目录
                    fileDir.mkdirs();
                }
                List<ProductAttrDO> productAttrDOList = productAttrService.getProductAttrList();
                Map<String, String> idNameMap = productAttrDOList.stream()
                        .collect(Collectors.toMap(attr -> String.valueOf(attr.getId()), ProductAttrDO::getAttrName));
                ExcelWriter excelWriter = EasyExcel.write(dir + fileName).withTemplate(inputStream).build();
                WriteSheet sheet = EasyExcel.writerSheet(0).build();
                excelWriter.fill(headMap, sheet);
                if (box.getLdStatus() != null && box.getLdStatus()>=43) {//43、已装柜、待封柜
                    //以secId排序
                    goodsItemList.sort(Comparator.comparing(BoxPreloadGoodsBackVO::getSecId).thenComparing(BoxPreloadGoodsBackVO::getTidanNum).thenComparing(BoxPreloadGoodsBackVO::getId));
                    int allTotalNum = 0;
                    BigDecimal allTotalVolume = new BigDecimal(0);
                    BigDecimal allTotalWeight = new BigDecimal(0);
                    BigDecimal allTotalWorth = new BigDecimal(0);
                    List<Long> secCount = goodsItemList.stream().map(BoxPreloadGoodsBackVO::getSecId).distinct().sorted().collect(Collectors.toList());
                    AtomicInteger colorRowNum = new AtomicInteger(6);
                    if (!CollectionUtils.isEmpty(secCount)) {
                        for (int i=0;i<secCount.size();i++){
                            List<String> list1 = new ArrayList();
                            List<List<String>> headList = new ArrayList<>();
                            String head = "";
                            if (i == 0) {
                                head = "第"+ NumberUtils.int2chineseNum(i+1) +"部分(Be loaded inside of the container)";
                            } else {
                                head = "第"+NumberUtils.int2chineseNum(i+1)+"部分(after "+ NumberUtils.format(i+1)+" parts)";
                            }
                            list1.add(head);
                            headList.add(list1);headList.add(list1);headList.add(list1);
                            headList.add(list1);headList.add(list1);headList.add(list1);
                            headList.add(list1);

                            headList.add(list1);headList.add(list1);headList.add(list1);
                            headList.add(list1);headList.add(list1);headList.add(list1);
                            headList.add(list1);

                            headList.add(list1);headList.add(list1);headList.add(list1);
//                            headList.add(list1);headList.add(list1);headList.add(list1);
                            headList.add(list1);headList.add(list1);

                            List<LoadDto> list = new ArrayList<>();

                            int totalNum = 0;
                            BigDecimal totalVolume = new BigDecimal(0);
                            BigDecimal totalWeight = new BigDecimal(0);
                            BigDecimal totalWorth = new BigDecimal(0);
                            List<Integer> mergeCountList = new ArrayList();
                            List<Integer> colorRowList = new ArrayList<>();
                            List<Integer> colorRowList2 = new ArrayList<>();
                            Map<String,Integer> numMap = new HashMap();
                            Map<String,Integer> colorMap = new HashMap<>();
                            Map<String,Integer> colorMap2 = new HashMap<>();

                            Long part = secCount.get(i);
                            int colorIndex = 0;

                            //以订单ID分组
                            Map<Long, List<BoxPreloadGoodsBackVO>> orderMap = goodsItemList.stream().filter(item -> item.getSecId().longValue()==part.longValue() && StringUtils.isNotBlank(item.getWarehouseInInfo()))
                                    .collect(Collectors.groupingBy(BoxPreloadGoodsBackVO::getOrderId,LinkedHashMap::new,Collectors.toList()));
                            for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : orderMap.entrySet()) {
                                boolean prodAttrTag = false;
                                Long orderId = entry.getKey();
                                OrderBackInfoDto orderInfo = orderQueryService.info(orderId);
                                List<BoxPreloadGoodsBackVO> boxPreload = entry.getValue();
                                List<OrderItemDO> orderItems = boxPreload.stream().map(item -> orderItemService.getOrderItem(item.getOrderItemId())).collect(Collectors.toList());
                                for (OrderItemDO orderItem : orderItems) {
                                    String attrNames = getAttrNameByIds(orderItem.getProdAttrIds(), idNameMap);
                                    if (attrNames.contains("危险品")){
                                        prodAttrTag = true;
                                    }
                                    List<OrderWarehouseInDO> orderWarehouseInDOS = orderWarehouseInService.selectList(OrderWarehouseInDO::getOrderItemId, orderItem.getOrderItemId());
                                    for (OrderWarehouseInDO orderWarehouseInDO : orderWarehouseInDOS) {
                                        List<OrderWarehouseInDetailsVO> warehouseInDetailsVOLists = orderWarehouseInDO.getOrderWarehouseInDetailsVOList();
                                        for (OrderWarehouseInDetailsVO warehouseInDetailsVOList : warehouseInDetailsVOLists) {
                                            String attrNames2 = getAttrNameByIds(warehouseInDetailsVOList.getProdAttrIds(), idNameMap);
                                            if (attrNames2.contains("危险品")){
                                                prodAttrTag = true;
                                            }
                                        }
                                    }
                                }

                                for(OrderItemDO item : orderItems) {
                                    LoadDto preloadDto = new LoadDto();
                                    String attrNames = getAttrNameByIds(item.getProdAttrIds(), idNameMap);
                                    preloadDto.setProdAttr(attrNames);
                                    // 当订单的报关方式=混合报关，订单的底色为蓝色，#0000FF
                                    // 当订单的报关方式=自单代报，订单的底色为橙色， #FF7F00
                                    // 当订单的商品包含了特性=危险品DG，订单的底色为黄色， #FFFF00
                                    // 当订单的商品包含了特性=危险品DG&订单的报关方式=自单代报/混合报关，底色为绿色，#00FF00
                                    if(orderInfo.getCustomsType()!=null){
                                        preloadDto.setCustomsType(DictFrameworkUtils.getDictDataFromCache("customs_type",orderInfo.getCustomsType()+"").getLabel());
                                        if (CustomsTypeEnum.CUSTOMS_TYPE_3.getValue().equals(orderInfo.getCustomsType())){
                                            Integer color = Integer.valueOf(IndexedColors.ROYAL_BLUE.index);
                                            if (prodAttrTag){
                                                color=Integer.valueOf(IndexedColors.BRIGHT_GREEN.index);
                                            }
                                            int andIncrement2 = colorRowNum.get();
                                            colorMap2.put(String.valueOf(andIncrement2),Integer.valueOf(color));
                                            colorMap.put(String.valueOf(andIncrement2),Integer.valueOf(color));
                                            colorRowList.add(andIncrement2);
                                            colorRowList2.add(andIncrement2);
                                        }else if (CustomsTypeEnum.CUSTOMS_TYPE_2.getValue().equals(orderInfo.getCustomsType())){
                                            Integer color = Integer.valueOf(IndexedColors.ORANGE.index);
                                            if (prodAttrTag){
                                                color=Integer.valueOf(IndexedColors.BRIGHT_GREEN.index);
                                            }
                                            int andIncrement2 = colorRowNum.get();
                                            colorMap2.put(String.valueOf(andIncrement2),Integer.valueOf(color));
                                            colorMap.put(String.valueOf(andIncrement2),Integer.valueOf(color));
                                            colorRowList.add(andIncrement2);
                                            colorRowList2.add(andIncrement2);
                                        }else {
                                            if (prodAttrTag){
                                                int andIncrement2 = colorRowNum.get();
                                                colorMap2.put(String.valueOf(andIncrement2),Integer.valueOf(IndexedColors.YELLOW.index));
                                                colorMap.put(String.valueOf(andIncrement2),Integer.valueOf(IndexedColors.YELLOW.index));
                                                colorRowList.add(andIncrement2);
                                                colorRowList2.add(andIncrement2);
                                            }
                                        }
                                    }

                                    List<OrderWarehouseInDO> orderWarehouseInDOS = orderWarehouseInService.selectList(OrderWarehouseInDO::getOrderItemId, item.getOrderItemId());
                                    boolean addColorFlag = true;
                                    for (OrderWarehouseInDO orderWarehouseInDO : orderWarehouseInDOS) {
                                        if (CollectionUtil.isNotEmpty(orderWarehouseInDO.getOrderWarehouseInDetailsVOList())) {
                                            int andIncrement = colorRowNum.getAndIncrement();
                                            colorMap.put(String.valueOf(andIncrement),colorList.get(colorIndex));
                                            colorRowList.add(andIncrement);//这里加1是为了设置颜色时过滤掉入仓记录那条
                                            addColorFlag = false;
                                            break;
                                        }
                                    }
                                    int mergeCount = 1;
                                    int mixedMergeCount = 1;
                                    preloadDto.setOrderNo(orderInfo.getOrderNo());
                                    if (numMap.get(orderInfo.getOrderNo()) != null) {
                                        preloadDto.setId(String.valueOf(numMap.get(orderInfo.getOrderNo())));
                                    } else {
                                        preloadDto.setId(String.valueOf(numMap.size()+1));
                                        numMap.put(orderInfo.getOrderNo(),numMap.size()+1);
                                    }
                                    preloadDto.setMarks(orderInfo.getMarks());
                                    List<OrderGuanlianDO> guanlianList = orderGuanlianService.getGuanlianListByOrderId(orderInfo.getOrderId());
                                    //rIdList为此订单的所有关联订单
                                    List<Long> rIdList = new ArrayList<>();
                                    for (OrderGuanlianDO orderGuanlianDO : guanlianList) {
                                        Long oId = orderGuanlianDO.getOrderId();
                                        Long rId = orderGuanlianDO.getRelateOrderId();
                                        if(oId.equals(orderId)) {
                                            rIdList.add((rId));
                                        } else if(rId.equals(orderId)) {
                                            rIdList.add(oId);
                                        }
                                    }
                                    if (CollectionUtil.isNotEmpty(rIdList)) {
                                        List<OrderDO> orderDOS = orderService.selectList(OrderDO::getOrderId, rIdList);
                                        if (CollectionUtil.isNotEmpty(orderDOS)) {
                                            preloadDto.setRelateOrder(orderDOS.stream().map(OrderDO::getOrderNo).collect(Collectors.joining(",")));
                                        }
                                    }
                                    preloadDto.setProdTitleZh(item.getProdTitleZh()+ " " +item.getProdTitleEn());
                                    preloadDto.setMerge(orderInfo.getOldNumbers());
                                    ProductBrankDO productBrank = productBrankService.getProductBrank(item.getBrand());
                                    if(productBrank!=null && StringUtils.isNotBlank(productBrank.getTitleZh())){
                                        preloadDto.setBrand(productBrank.getTitleZh());
                                    }
                                    if(StringUtils.isNotBlank(item.getMaterial())){
                                        preloadDto.setMaterial(DictFrameworkUtils.getDictDataFromCache("product_material",item.getMaterial()+"").getLabel());
                                    }
                                    if(productBrank !=null && productBrank.getFiling()!=null){
                                        preloadDto.setProductRecord(productBrank.getFiling().equals("0")? "N":"Y");
                                    }
                                    preloadDto.setSumNum(item.getWarehouseInInfoVO().getCartonsNum());
                                    if(CollectionUtil.isNotEmpty(orderWarehouseInDOS)){
                                        String warehousingSpecificationType = orderWarehouseInDOS.stream().map(j -> {
                                            return DictFrameworkUtils.getDictDataFromCache("warehousing_specification_type", j.getSpecificationType() + "").getLabel();
                                        }).distinct().collect(Collectors.joining("/"));
                                        String packagingType = orderWarehouseInDOS.stream().map(j -> {
                                            return DictFrameworkUtils.getDictDataFromCache("packaging_type", j.getUnit()).getLabel()+"/"+DictFrameworkUtils.getDictDataFromCache("packaging_type", j.getUnit()).getLabelEn();
                                        }).distinct().collect(Collectors.joining("/"));
                                        preloadDto.setUnit(warehousingSpecificationType);
                                        preloadDto.setPackageType(packagingType);
                                    }
                                    preloadDto.setSumVolume(item.getWarehouseInInfoVO().getVolume());
                                    preloadDto.setUsageIds(getUsageMsg(item.getUsageIds()));
                                    preloadDto.setSumWeight(item.getWarehouseInInfoVO().getWeight());

                                    totalWorth = totalWorth.add(item.getWorth());
                                    preloadDto.setWorth(item.getWorth());
                                    preloadDto.setQuantity(item.getWarehouseInInfoVO().getQuantityAll());
                                    list.add(preloadDto);
                                    if (CollectionUtil.isNotEmpty(orderWarehouseInDOS)) {
                                        for (OrderWarehouseInDO j : orderWarehouseInDOS) {
                                            if (CollectionUtil.isNotEmpty(j.getOrderWarehouseInDetailsVOList())) {
                                                mergeCount = mergeCount + j.getOrderWarehouseInDetailsVOList().size();
                                                for (OrderWarehouseInDetailsVO k : j.getOrderWarehouseInDetailsVOList()) {
                                                    int andIncrement1 = colorRowNum.getAndIncrement();
                                                    colorMap.put(String.valueOf(andIncrement1),colorList.get(colorIndex));
                                                    colorRowList.add(andIncrement1);
                                                    LoadDto preloadDto1 = new LoadDto();
                                                    BeanUtil.copyProperties(preloadDto,preloadDto1);
                                                    ProductBrankDO productBrankDetail = productBrankService.getProductBrank(k.getBrand());
                                                    if(productBrankDetail!=null && StringUtils.isNotBlank(productBrankDetail.getTitleZh())){
                                                        preloadDto1.setBrand(productBrankDetail.getTitleZh());
                                                    }
                                                    if(productBrank !=null && productBrank.getFiling()!=null){
                                                        preloadDto1.setProductRecord(productBrank.getFiling().equals("0")? "N":"Y");
                                                    }
                                                    preloadDto1.setQuantity(k.getQuantityAll());
                                                    if (StringUtils.isNotBlank(k.getMaterial())) {
                                                        preloadDto1.setMaterial(DictFrameworkUtils.getDictDataFromCache("product_material",k.getMaterial()+"").getLabel());
                                                    }
                                                    preloadDto1.setProdAttr(getAttrNameByIds(k.getProdAttrIds(),idNameMap));
                                                    preloadDto1.setUsageIds(getUsageMsg(k.getUsageIds()));
                                                    ProductDO product = productService.getProduct(k.getProdId());
                                                    if (product != null) {
                                                        preloadDto1.setProdTitleZh(product.getTitleZh() + "" + product.getTitleEn());
                                                    }
                                                    list.add(preloadDto1);
                                                }
                                            }
                                        }
                                    }
                                    mergeCountList.add(mergeCount);
                                    if (colorIndex == 7) {
                                        colorIndex = 0;
                                    } else {
                                        colorIndex ++;
                                    }
                                    totalNum = totalNum + item.getWarehouseInInfoVO().getCartonsNum();
                                    totalVolume = totalVolume.add(item.getWarehouseInInfoVO().getVolume());
                                    totalWeight = totalWeight.add(item.getWarehouseInInfoVO().getWeight());
                                    if (addColorFlag) {
                                        colorRowNum.getAndIncrement();
                                    }
                                }
                            }
                            int andIncrement = colorRowNum.getAndIncrement();
                            colorMap2.put(String.valueOf(andIncrement),Integer.valueOf(IndexedColors.YELLOW.index));
                            colorMap.put(String.valueOf(andIncrement),Integer.valueOf(IndexedColors.YELLOW.index));

                            colorRowList.add(andIncrement);
                            colorRowList2.add(andIncrement);
                            colorRowNum.getAndAdd(1);
                            LoadDto preloadDto = new LoadDto();
                            preloadDto.setId("TOTAL");
                            preloadDto.setSumNum(totalNum);
                            preloadDto.setSumVolume(totalVolume);
                            preloadDto.setSumWeight(totalWeight);
                            preloadDto.setWorth(totalWorth);
                            list.add(preloadDto);
                            allTotalNum = allTotalNum + totalNum;
                            allTotalVolume = allTotalVolume.add(totalVolume);
                            allTotalWeight = allTotalWeight.add(totalWeight);
                            allTotalWorth = allTotalWorth.add(totalWorth);

                            if(i==secCount.size()-1){
                                LoadDto preloadDto1 = new LoadDto();
                                preloadDto1.setId("IN TOTAL");
                                preloadDto1.setProdAttr("合计");
                                preloadDto1.setSumNum(allTotalNum);
                                preloadDto1.setSumVolume(allTotalVolume);
                                preloadDto1.setSumWeight(allTotalWeight);
                                preloadDto1.setWorth(allTotalWorth);
                                list.add(preloadDto1);
                                colorMap2.put(String.valueOf(andIncrement+1),Integer.valueOf(IndexedColors.YELLOW.index));
                                colorMap.put(String.valueOf(andIncrement+1),Integer.valueOf(IndexedColors.YELLOW.index));
                                colorRowList.add(andIncrement+1);
                                colorRowList2.add(andIncrement+1);
                            }

                            WriteTable writeTable = EasyExcel.writerTable(i)
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,0,1,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,1,1,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,2,1,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,3, colorMap))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,4, colorMap))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,5, colorMap))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,6, colorMap))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,7, colorMap))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,8,2,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,9,1,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,10,2,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,11,2,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,12,1,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,13, colorMap))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,14, colorMap))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,15,1,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,16,1,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,17,1,colorRowList2,colorMap2))
                                    .registerWriteHandler(new CustomLoadMergeStrategyNew(list,mergeCountList,18,2,colorRowList2,colorMap2))
                                    .automaticMergeHead(true).head(headList).needHead(true).build();
                            excelWriter.write(list,sheet,writeTable);
                        }
                    }

                }
                excelWriter.finish();
                inputStream.close();
                // 获取到临时文件
                File file = new File(dir + fileName);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());
            } catch (Exception e) {

                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }

    private String getUsageMsg(String usageIdsDetail) {
        if (StringUtils.isNotEmpty(usageIdsDetail)) {
            StringJoiner labelJoiner = new StringJoiner(",");
            for (String usageId : usageIdsDetail.split(",")) {
                DictDataRespDTO usageValue = DictFrameworkUtils.getDictDataFromCache("order_item_usage", usageId);
                if (usageValue != null) {
                    labelJoiner.add(usageValue.getLabel());
                }
            }
            return labelJoiner.toString();
        } else {
            return "";
        }
    }

    private String getAttrNameByIds(String attrIds, Map<String, String> idNameMap) {
        StringJoiner labelJoiner = new StringJoiner(",");
        if(StringUtils.isNotBlank(attrIds)) {
            String[] attrIdArray = attrIds.split(",");
            for (String attrId : attrIdArray) {
                //map一定包含数据，否则是数据错误
                labelJoiner.add(idNameMap.get(attrId));
            }
            return labelJoiner.toString();
        }
        return "";
    }
}
